(ns clojsa.handler
  (:require [clojsa.views :as views]
            [clojsa.saclient :as client]
            [clojsa.saparser :as parser]
            [compojure.core :refer :all]
            [compojure.route :as route]
            [compojure.coercions :refer [as-int]]
            [ring.middleware.defaults :refer [wrap-defaults site-defaults]]
            [ring.middleware.session :refer [wrap-session]]
            [ring.middleware.session.cookie :refer (cookie-store)]))


(defn get-thread [session id page]
  (let [turl (client/thread-url id page)
        tresp (client/thread-response session turl)]
    (parser/thread-map id page tresp)))

(defroutes app-routes

  (GET "/" request (views/index-page request))

  (GET "/login" [] (views/login-page))
  (POST "/login" [username password :as {session :session}]
        (let [resp-cookies (client/login-response username password session)]
          {:headers {"Content-Type" "text/plain; charset=utf-8"}
           :body (with-out-str (clojure.pprint/pprint resp-cookies))
           :session (assoc session :cookies resp-cookies)}))

  (GET "/thread/:id" [id :<< as-int
                      page :<< as-int
                      :as {session :session}]
       (let [thread (get-thread session id page)]
         (views/thread-page thread)))

  (route/not-found "Not Found"))

(def app
  (-> app-routes
      (wrap-defaults site-defaults)
      (wrap-session {:cookie-attrs {:max-age (* 7 24 3600)}
                     :store (cookie-store {:key "699a7e4bd79b3f56"})})))
