(ns clojsa.saparser
  (:require [clojsa.saclient :refer [thread-url thread-response]]
            [clojure.string :as string]
            [hickory.core :refer :all]
            [hickory.select :as s]
            [hickory.convert :refer [hickory-to-hiccup]]))

(defn hickory-doc [doc]
  (-> doc parse as-hickory))

(defn parse-title [htree]
  (-> (s/select (s/child (s/tag :title)) htree)
      first :content first
      (string/replace #" - The Something Awful Forums" "")))

(defn parse-pagecount [htree]
  (-> (s/select (s/descendant
                 (s/class :pages) (s/tag :option)) htree)
      last :content first Integer/parseInt))

(defn parse-thread [htree]
  (-> (s/select (s/descendant
                 (s/id :thread))
                htree)
      first))

(defn select-td [class-key htree]
  (s/select (s/descendant
             (s/and (s/tag :td) (s/class class-key))) htree))

(defn parse-ui [ui]
  (let [ui (first (s/select (s/descendant (s/tag :dl)) ui))]
    (hickory-to-hiccup ui)))

(defn parse-pd [pd]
  (string/trim (last (hickory-to-hiccup pd))))

(defn hickory-div [class content]
  {:type :element,
  :attrs {:class class},
  :tag :div,
  :content content})

(defn parse-pb [pb]
  (let [pb (-> pb :content)]
    (hickory-to-hiccup (hickory-div "postbody" pb))))

(defn thread-map [id page doc]
  (let [htree       (hickory-doc doc)
        title       (parse-title htree)
        page-count  (parse-pagecount htree)
        thread-tree (parse-thread htree)
        userinfo    (select-td :userinfo thread-tree)
        postdate    (select-td :postdate thread-tree)
        postbody    (select-td :postbody thread-tree)]
    {:title title
     :id id
     :page page
     :page-count page-count
     :content
     (for [[ui pd pb] (partition 3 (interleave userinfo postdate postbody))
           :when (not= "Adbot" (-> (s/select (s/child (s/class :author)) ui)
                                   first :content first))]
       {:ui (parse-ui ui) :pd (parse-pd pd) :pb (parse-pb pb)})}))
